@@ -3,8 +3,8 @@
  * LUKS - Linux Unified Key Setup
  *
  * Copyright (C) 2004-2006 Clemens Fruhwirth <clemens@endorphin.org>
- * Copyright (C) 2009-2024 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2012-2024 Milan Broz
+ * Copyright (C) 2009-2025 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2012-2025 Milan Broz
  */
 
 #include <stdio.h>
@@ -18,7 +18,8 @@
 static void _error_hint(struct crypt_device *ctx, const char *device,
 			const char *cipher, const char *mode, size_t keyLength)
 {
-	char *c, cipher_spec[MAX_CIPHER_LEN * 3];
+	const char *c;
+	char cipher_spec[MAX_CIPHER_LEN * 3];
 
 	if (snprintf(cipher_spec, sizeof(cipher_spec), "%s-%s", cipher, mode) < 0)
 		return;
@@ -88,7 +89,7 @@ static int LUKS_endec_template(char *src, size_t srcLength,
 
 	r = dm_crypt_target_set(&dmd.segment, 0, dmd.size,
 			crypt_metadata_device(ctx), vk, cipher_spec, 0, sector,
-			NULL, 0, SECTOR_SIZE);
+			NULL, 0, 0, SECTOR_SIZE);
 	if (r)
 		goto out;
 
@@ -96,7 +97,7 @@ static int LUKS_endec_template(char *src, size_t srcLength,
 	if (r < 0) {
 		if (r != -EACCES && r != -ENOTSUP)
 			_error_hint(ctx, device_path(crypt_metadata_device(ctx)),
-				    cipher, cipher_mode, vk->keylength * 8);
+				    cipher, cipher_mode, crypt_volume_key_length(vk) * 8);
 		r = -EIO;
 		goto out;
 	}
@@ -140,7 +141,8 @@ int LUKS_encrypt_to_storage(char *src, size_t srcLength,
 		return -EINVAL;
 
 	/* Encrypt buffer */
-	r = crypt_storage_init(&s, SECTOR_SIZE, cipher, cipher_mode, vk->key, vk->keylength, false);
+	r = crypt_storage_init(&s, SECTOR_SIZE, cipher, cipher_mode, crypt_volume_key_get_key(vk),
+			crypt_volume_key_length(vk), false);
 
 	if (r)
 		log_dbg(ctx, "Userspace crypto wrapper cannot use %s-%s (%d).",
@@ -153,7 +155,7 @@ int LUKS_encrypt_to_storage(char *src, size_t srcLength,
 
 	if (r) {
 		_error_hint(ctx, device_path(device), cipher, cipher_mode,
-			    vk->keylength * 8);
+			    crypt_volume_key_length(vk) * 8);
 		return r;
 	}
 
@@ -205,7 +207,8 @@ int LUKS_decrypt_from_storage(char *dst, size_t dstLength,
 	if (MISALIGNED_512(dstLength))
 		return -EINVAL;
 
-	r = crypt_storage_init(&s, SECTOR_SIZE, cipher, cipher_mode, vk->key, vk->keylength, false);
+	r = crypt_storage_init(&s, SECTOR_SIZE, cipher, cipher_mode, crypt_volume_key_get_key(vk),
+			crypt_volume_key_length(vk), false);
 
 	if (r)
 		log_dbg(ctx, "Userspace crypto wrapper cannot use %s-%s (%d).",
@@ -218,7 +221,7 @@ int LUKS_decrypt_from_storage(char *dst, size_t dstLength,
 
 	if (r) {
 		_error_hint(ctx, device_path(device), cipher, cipher_mode,
-			    vk->keylength * 8);
+			   crypt_volume_key_length(vk) * 8);
 		return r;
 	}
 
