@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * LUKS keyslot entropy tester. Works only for header version 1.
  *
@@ -13,20 +14,6 @@
  *    v0.2: 08.10.2012 Converted to use libcryptsetup
  *
  * Copyright (C) 2012, Arno Wagner <arno@wagner.name>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <stdlib.h>
@@ -45,7 +32,7 @@ const char *help =
 "\n"
 "This tool checks all keyslots of a LUKS device for \n"
 "low entropy sections. If any are found, they are reported. \n"
-"This allows to find areas damaged by things like filesystem \n"
+"This allows one to find areas damaged by things like filesystem \n"
 "creation or RAID superblocks. \n"
 "\n"
 "Options: \n"
@@ -61,7 +48,7 @@ const char *help =
 "            the threshold down to reduce misdetection. For values\n"
 "            larger than the default you need to adjust the threshold\n"
 "            up to retain sensitivity.\n"
-"  -v        Print found suspicuous sectors verbosely. \n"
+"  -v        Print found suspicious sectors verbosely. \n"
 "  -d        Print decimal addresses instead of hex ones.\n"
 "\n";
 
@@ -174,7 +161,7 @@ static int check_keyslots(FILE *out, struct crypt_device *cd, int f_luks)
 	crypt_keyslot_info ki;
 	unsigned char buffer[sector_size];
 
-	for (i = 0; i < crypt_keyslot_max(CRYPT_LUKS1) ; i++) {
+	for (i = 0; i < crypt_keyslot_max(crypt_get_type(cd)) ; i++) {
 		fprintf(out, "- processing keyslot %d:", i);
 		ki = crypt_keyslot_status(cd, i);
 		if (ki == CRYPT_SLOT_INACTIVE) {
@@ -321,8 +308,8 @@ int main(int argc, char **argv)
 	device = argv[optind];
 
 	/* test whether we can open and read device */
-	/* This is neded as we are reading the actual data
-	* in the keyslots dirtectly from the LUKS container.
+	/* This is needed as we are reading the actual data
+	* in the keyslots directly from the LUKS container.
 	*/
 	f_luks = open(device, O_RDONLY);
 	if (f_luks == -1) {
@@ -351,7 +338,7 @@ int main(int argc, char **argv)
 	 * This should also make sure a valid LUKS1 header is on disk
 	 * and hence we should be able to skip magic and version checks.
 	 */
-	res = crypt_load(cd, CRYPT_LUKS1, NULL);
+	res = crypt_load(cd, CRYPT_LUKS, NULL);
 	if (res < 0) {
 		fprintf(stderr, "crypt_load() failed. LUKS header too broken/absent?\n");
 		crypt_free(cd);
