@@ -1,24 +1,11 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * cryptsetup - setup cryptographic volumes for dm-crypt
  *
  * Copyright (C) 2004 Jana Saout <jana@saout.de>
  * Copyright (C) 2004-2007 Clemens Fruhwirth <clemens@endorphin.org>
- * Copyright (C) 2009-2024 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2009-2024 Milan Broz
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * Copyright (C) 2009-2025 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2025 Milan Broz
  */
 
 #ifndef CRYPTSETUP_H
@@ -103,6 +90,7 @@ struct tools_progress_params {
 
 int tools_progress(uint64_t size, uint64_t offset, void *usrptr);
 const char *tools_get_device_name(const char *device, char **r_backing_file);
+int tools_check_newname(const char *name);
 
 int tools_read_vk(const char *file, char **key, int keysize);
 int tools_write_mk(const char *file, const char *key, int keysize);
@@ -129,6 +117,13 @@ int tools_lookup_crypt_device(struct crypt_device *cd, const char *type,
 /* each utility is required to implement it */
 void tools_cleanup(void);
 
+/* keyring helpers */
+int tools_parse_vk_description(const char *key_description, char **ret_key_description);
+int tools_parse_vk_and_keyring_description(
+	struct crypt_device *cd,
+	char **keyring_key_descriptions,
+	int keyring_key_links_count);
+
 /* Log */
 #define log_dbg(x...) crypt_logf(NULL, CRYPT_LOG_DEBUG, x)
 #define log_std(x...) crypt_logf(NULL, CRYPT_LOG_NORMAL, x)
