@@ -15,16 +15,7 @@
 # $1 full target dir
 # $2 full source luks2 image
 
-function prepare()
-{
-	cp $SRC_IMG $TGT_IMG
-	test -d $TMPDIR || mkdir $TMPDIR
-	read_luks2_json0 $TGT_IMG $TMPDIR/json0
-	read_luks2_bin_hdr0 $TGT_IMG $TMPDIR/hdr0
-	read_luks2_bin_hdr1 $TGT_IMG $TMPDIR/hdr1
-}
-
-function generate()
+generate()
 {
 	# 2 MiB metadata
 	TEST_MDA_SIZE=$LUKS2_HDR_SIZE_2M
@@ -44,32 +35,19 @@ function generate()
 	test ${#json_str} -lt $((LUKS2_JSON_SIZE*512)) || exit 2
 
 	write_luks2_json "$json_str" $TMPDIR/json0 $TEST_JSN_SIZE
+	write_luks2_json "$json_str" $TMPDIR/json1 $TEST_JSN_SIZE
 
 	write_bin_hdr_size $TMPDIR/hdr0 $TEST_MDA_SIZE_BYTES
 	write_bin_hdr_size $TMPDIR/hdr1 $TEST_MDA_SIZE_BYTES
 
-	merge_bin_hdr_with_json $TMPDIR/hdr0 $TMPDIR/json0 $TMPDIR/area0 $TEST_JSN_SIZE
-	merge_bin_hdr_with_json $TMPDIR/hdr1 $TMPDIR/json0 $TMPDIR/area1 $TEST_JSN_SIZE
-
-	erase_checksum $TMPDIR/area0
-	chks0=$(calc_sha256_checksum_file $TMPDIR/area0)
-	write_checksum $chks0 $TMPDIR/area0
-
-	erase_checksum $TMPDIR/area1
-	chks0=$(calc_sha256_checksum_file $TMPDIR/area1)
-	write_checksum $chks0 $TMPDIR/area1
-
-	kill_bin_hdr $TMPDIR/area1
-
-	write_luks2_hdr0 $TMPDIR/area0 $TGT_IMG $TEST_MDA_SIZE
-	write_luks2_hdr1 $TMPDIR/area1 $TGT_IMG $TEST_MDA_SIZE
+	lib_mangle_json_hdr0 $TEST_MDA_SIZE $TEST_JSN_SIZE
+	lib_mangle_json_hdr1 $TEST_MDA_SIZE $TEST_JSN_SIZE kill
 }
 
-function check()
+check()
 {
-	read_luks2_bin_hdr1 $TGT_IMG $TMPDIR/hdr_res1 $TEST_MDA_SIZE
-	local str_res1=$(head -c 6 $TMPDIR/hdr_res1)
-	test "$str_res1" = "VACUUM" || exit 2
+	lib_hdr1_killed $TEST_MDA_SIZE || exit 2
+
 	read_luks2_json0 $TGT_IMG $TMPDIR/json_res0 $TEST_JSN_SIZE
 	jq -c --arg koff $KEYSLOTS_OFFSET --arg jsize $JSON_SIZE \
 		'if ([.keyslots[].area.offset] | map(tonumber) | min | tostring != $koff) or
@@ -77,18 +55,7 @@ function check()
 		then error("Unexpected value in result json") else empty end' $TMPDIR/json_res0 || exit 5
 }
 
-function cleanup()
-{
-	rm -f $TMPDIR/*
-	rm -fd $TMPDIR
-}
-
-test $# -eq 2 || exit 1
-
-TGT_IMG=$1/$(test_img_name $0)
-SRC_IMG=$2
-
-prepare
+lib_prepare $@
 generate
 check
-cleanup
+lib_cleanup
